#!/usr/bin/env python
# -*- coding: UTF-8 -*-

# Copyright (c) 2020, Sandflow Consulting LLC
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
# 1. Redistributions of source code must retain the above copyright notice, this
#    list of conditions and the following disclaimer.
# 2. Redistributions in binary form must reproduce the above copyright notice,
#    this list of conditions and the following disclaimer in the documentation
#    and/or other materials provided with the distribution.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
# ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
# DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
# ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
# (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
# ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

"""SCC caption line"""

from __future__ import annotations

import logging
from typing import List, Union

from ttconv.scc.caption_text import SccCaptionText

LOGGER = logging.getLogger(__name__)


class SccCaptionLine:
  """Caption paragraph line"""

  @staticmethod
  def default():
    """Initializes a default caption paragraph line"""
    return SccCaptionLine(0, 0)

  def __init__(self, row: int, indent: int):
    self._row: int = row  # Row in the active area
    self._indent: int = indent  # Indentation in the active area

    self._cursor: int = 0  # Position of the cursor on the line
    self._current_text: SccCaptionText = SccCaptionText()  # Text content where the cursor is
    self._texts: List[SccCaptionText] = [self._current_text]

  def add_text(self, text: Union[SccCaptionText, str]):
    """Add text to line"""

    if isinstance(text, SccCaptionText):
      self._texts.append(text)
      self._current_text = self._texts[-1]
      self._cursor = self.get_length()

    elif isinstance(text, str):
      remaining_text = text

      # While the cursor is not on the last text element, and some text remains
      while self._current_text is not self._texts[-1] and len(remaining_text) > 0:
        available = self._current_text.get_length() - self._current_text.get_cursor()
        text_to_write = remaining_text[:available]

        # Replace current text element content
        self._append_text(text_to_write)
        remaining_text = remaining_text[available:]

      # If some text remains on the last text element
      if len(remaining_text) > 0:
        assert self._current_text is self._texts[-1]

        # Replace and append to current text element content
        self._append_text(remaining_text)

    else:
      raise ValueError("Unsupported text type for SCC caption line")

  def _append_text(self, text: str):
    """Appends text and update cursor position"""
    self._current_text.append(text)
    if self._cursor < 0:
      self._cursor = 0

    self.set_cursor(self._cursor + len(text))

  def indent(self, indent: int):
    """Indent current line"""
    self._indent += indent

  def get_current_text(self) -> SccCaptionText:
    """Returns current text content"""
    return self._current_text

  def get_texts(self) -> List[SccCaptionText]:
    """Returns the text contents of the line"""
    return self._texts

  def get_length(self) -> int:
    """Returns the total text length"""
    return sum([text.get_length() for text in self._texts])

  def set_cursor(self, column: int):
    """Set cursor position"""
    if column > self.get_length():
      LOGGER.warning("Expected cursor position is beyond the line length: force cursor at the end of the line.")
      column = self.get_length()

    self._cursor = column

    diff = self._cursor
    for index, text in enumerate(self._texts):
      is_last_text = index == len(self._texts) - 1
      if diff > text.get_length() or (diff == text.get_length() and not is_last_text):
        diff -= text.get_length()
      else:
        self._current_text = text
        self._current_text.set_cursor_at(diff)
        return

  def get_cursor(self) -> int:
    """Returns the cursor position in the line"""
    return self._cursor

  def set_row(self, row: int):
    """Sets the line row"""
    self._row = row

  def get_row(self) -> int:
    """Returns the line row"""
    return self._row

  def get_indent(self) -> int:
    """Returns the line indentation"""
    return self._indent

  def clear(self):
    """Clears the line text contents"""
    self._texts.clear()
    self._current_text = SccCaptionText()
    self._texts = [self._current_text]
    self.set_cursor(0)

  def is_empty(self) -> bool:
    """Returns whether the line text is empty or not"""
    return self.get_length() == 0

  def get_leading_spaces(self) -> int:
    """Returns the number of leading space characters of the line"""
    index = 0
    leading_spaces = 0

    while index < len(self.get_texts()):
      first_text = self.get_texts()[index].get_text()
      if first_text.isspace():
        leading_spaces += len(first_text)
        index += 1
      else:
        break

    return leading_spaces + len(first_text) - len(first_text.lstrip())

  def get_trailing_spaces(self) -> int:
    """Returns the number of trailing space characters of the line"""
    index = 1
    trailing_spaces = 0
    last_text = self.get_texts()[-index].get_text()

    while last_text.isspace() and index < len(self.get_texts()):
      trailing_spaces += len(last_text)
      index += 1
      last_text = self.get_texts()[-index].get_text()

    return trailing_spaces + len(last_text) - len(last_text.rstrip())

  def __repr__(self):
    return "<" + self.__class__.__name__ + " " + str(self.__dict__) + ">"



#!/usr/bin/env python
# -*- coding: UTF-8 -*-

# Copyright (c) 2020, Sandflow Consulting LLC
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
# 1. Redistributions of source code must retain the above copyright notice, this
#    list of conditions and the following disclaimer.
# 2. Redistributions in binary form must reproduce the above copyright notice,
#    this list of conditions and the following disclaimer in the documentation
#    and/or other materials provided with the distribution.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
# ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
# DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
# ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
# (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
# ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

"""SCC disassembly functions"""
import logging

from ttconv.scc.codes.attribute_codes import SccAttributeCode
from ttconv.scc.codes.control_codes import SccControlCode
from ttconv.scc.codes.extended_characters import SccExtendedCharacter
from ttconv.scc.codes.mid_row_codes import SccMidRowCode
from ttconv.scc.codes.preambles_address_codes import SccPreambleAddressCode
from ttconv.scc.codes.special_characters import SccSpecialCharacter
from ttconv.scc.word import SccWord
from ttconv.style_properties import ColorType, NamedColors, FontStyleType, TextDecorationType

LOGGER = logging.getLogger(__name__)


def get_color_disassembly(color: ColorType) -> str:
  """Get color disassembly code"""
  disassembly = ""

  if color is None:
    return disassembly

  rgb_color = color.components[:-1]
  if rgb_color == NamedColors.white.value.components[:-1]:
    disassembly = "Wh"
  elif rgb_color == NamedColors.green.value.components[:-1]:
    disassembly = "Gr"
  elif rgb_color == NamedColors.blue.value.components[:-1]:
    disassembly = "Bl"
  elif rgb_color == NamedColors.cyan.value.components[:-1]:
    disassembly = "Cy"
  elif rgb_color == NamedColors.red.value.components[:-1]:
    disassembly = "R"
  elif rgb_color == NamedColors.yellow.value.components[:-1]:
    disassembly = "Y"
  elif rgb_color == NamedColors.magenta.value.components[:-1]:
    disassembly = "Ma"
  elif rgb_color == NamedColors.black.value.components[:-1]:
    disassembly = "Bk"

  # Alpha channel
  if color.components[3] == 0:
    disassembly = "T"
  elif color.components[3] == 0x88:
    disassembly += "S"
  elif color.components[3] != 0xFF:
    LOGGER.warning("Skip unsupported %s alpha level in disassembly format.", color.components[3])

  if not disassembly:
    LOGGER.warning("Unsupported '%s' color in disassembly format.", color.components)

  return disassembly


def get_font_style_disassembly(font_style: FontStyleType) -> str:
  """Get font style disassembly code"""
  if font_style is FontStyleType.italic:
    return "I"
  return ""


def get_text_decoration_disassembly(text_decoration: TextDecorationType) -> str:
  """Get text decoration disassembly code"""
  if text_decoration is not None and text_decoration.underline is True:
    return "U"
  return ""


def get_scc_word_disassembly(scc_word: SccWord, show_channel = False) -> str:
  """Returns the disassembly code for specified SCC word"""
  if scc_word.value == 0x0000:
    return "{}"

  if scc_word.byte_1 < 0x20:

    pac = SccPreambleAddressCode.find(scc_word.byte_1, scc_word.byte_2)

    if pac is not None:
      disassembly_code = "{"
      if show_channel:
        disassembly_code += str(pac.get_channel()) + "|"
      disassembly_code += f"{pac.get_row():02}"
      color = pac.get_color()
      indent = pac.get_indent()
      if indent is not None and indent > 0:
        disassembly_code += f"{indent :02}"
      elif color is not None:
        disassembly_code += get_color_disassembly(color)
        disassembly_code += get_font_style_disassembly(pac.get_font_style())
        disassembly_code += get_text_decoration_disassembly(pac.get_text_decoration())
      else:
        disassembly_code += "00"
      disassembly_code += "}"
      return disassembly_code

    attribute_code = SccAttributeCode.find(scc_word.value)

    if attribute_code is not None:
      disassembly_code = "{"
      if show_channel:
        disassembly_code += str(attribute_code.get_channel(scc_word.value)) + "|"
      disassembly_code += "B" if attribute_code.is_background() else ""
      disassembly_code += get_color_disassembly(attribute_code.get_color())
      disassembly_code += get_text_decoration_disassembly(attribute_code.get_text_decoration())
      disassembly_code += "}"
      return disassembly_code

    mid_row_code = SccMidRowCode.find(scc_word.value)

    if mid_row_code is not None:
      disassembly_code = "{"
      if show_channel:
        disassembly_code += str(mid_row_code.get_channel(scc_word.value)) + "|"
      disassembly_code += get_color_disassembly(mid_row_code.get_color())
      disassembly_code += get_font_style_disassembly(mid_row_code.get_font_style())
      disassembly_code += get_text_decoration_disassembly(mid_row_code.get_text_decoration())
      disassembly_code += "}"
      return disassembly_code

    control_code = SccControlCode.find(scc_word.value)

    if control_code is not None:
      disassembly_code = "{"
      if show_channel:
        disassembly_code += str(control_code.get_channel(scc_word.value)) + "|"
      disassembly_code += control_code.get_name()
      disassembly_code += "}"
      return disassembly_code

    # print(f"{hex(scc_word.byte_1)}{hex(scc_word.byte_2)}")
    spec_char = SccSpecialCharacter.find(scc_word.value)

    if spec_char is not None:
      if show_channel:
        disassembly_code = "["
        disassembly_code += str(spec_char.get_channel(scc_word.value))
        disassembly_code += "]"
        disassembly_code += spec_char.get_unicode_value()
        return disassembly_code
      return spec_char.get_unicode_value()

    extended_char = SccExtendedCharacter.find(scc_word.value)

    if extended_char is not None:
      if show_channel:
        disassembly_code = "["
        disassembly_code += str(extended_char.get_channel(scc_word.value))
        disassembly_code += "]"
        disassembly_code += extended_char.get_unicode_value()
        return disassembly_code
      return extended_char.get_unicode_value()

    LOGGER.warning("Unsupported SCC word: %s", hex(scc_word.value))
    return "{??}"

  return scc_word.to_text()



#!/usr/bin/env python
# -*- coding: UTF-8 -*-

# Copyright (c) 2020, Sandflow Consulting LLC
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
# 1. Redistributions of source code must retain the above copyright notice, this
#    list of conditions and the following disclaimer.
# 2. Redistributions in binary form must reproduce the above copyright notice,
#    this list of conditions and the following disclaimer in the documentation
#    and/or other materials provided with the distribution.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
# ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
# DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
# ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
# (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
# ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

'''Unit tests for the style properties defined in the canonical model'''

# pylint: disable=R0201,C0115,C0116

from fractions import Fraction
import unittest
from ttconv import model
from ttconv.isd import ISD
from ttconv.style_properties import ExtentType, FontVariantType, LengthType, NamedColors, StyleProperties, TextShadowType

class TestModelStyleProperties(unittest.TestCase):

  def setUp(self):
    self.doc = model.ContentDocument()

    self.r1 = model.Region("r1", self.doc)
    self.doc.put_region(self.r1)

    self.b = model.Body(self.doc)
    self.doc.set_body(self.b)

    self.div1 = model.Div(self.doc)
    self.div1.set_region(self.r1)
    self.b.push_child(self.div1)

    self.p1 = model.P(self.doc)
    self.div1.push_child(self.p1)

    self.span1 = model.Span(self.doc)
    self.p1.push_child(self.span1)

    self.text1 = model.Text(self.doc, "span1")
    self.span1.push_child(self.text1)

  def test_fontVariant(self):
    self.b.set_style(StyleProperties.FontVariant, FontVariantType.subscript)
    self.assertFalse(self.b.is_style_applicable(StyleProperties.FontVariant))

    a = model.DiscreteAnimationStep(StyleProperties.FontVariant, 1, None, FontVariantType.superscript)
    self.p1.add_animation_step(a)

    isd = ISD.from_model(self.doc, 0)
    r = list(isd)[0]
    self.assertIsNone(r[0][0].get_style(StyleProperties.FontVariant))
    self.assertEqual(r[0][0][0][0].get_style(StyleProperties.FontVariant), FontVariantType.subscript)

    isd = ISD.from_model(self.doc, 1)
    r = list(isd)[0]
    self.assertIsNone(r[0][0].get_style(StyleProperties.FontVariant))
    self.assertEqual(r[0][0][0][0].get_style(StyleProperties.FontVariant), FontVariantType.superscript)

  def test_make_initial(self):
    for style in StyleProperties.ALL:
      with self.subTest(style.__name__):
        style.validate(style.make_initial_value())

  def test_text_shadow_5_shadows_raises(self):
    span = model.Span()
    with self.assertRaises(ValueError):
      shadow = TextShadowType.Shadow(
        x_offset=LengthType(1, LengthType.Units.pct),
        y_offset=LengthType(1, LengthType.Units.pct),
      )
      span.set_style(
        StyleProperties.TextShadow,
        TextShadowType(shadows=(shadow, shadow, shadow, shadow, shadow)),
      )

  def test_extent_c_fail(self):
    doc = model.ContentDocument()

    r1 = model.Region("r1", doc)
    with self.assertRaises(ValueError):
      r1.set_style(
        StyleProperties.Extent,
        ExtentType(
          width=LengthType(10, LengthType.Units.c),
          height=LengthType(20, LengthType.Units.c)
        )
      )

if __name__ == '__main__':
  unittest.main()



#!/usr/bin/env python
# -*- coding: UTF-8 -*-

# Copyright (c) 2020, Sandflow Consulting LLC
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
# 1. Redistributions of source code must retain the above copyright notice, this
#    list of conditions and the following disclaimer.
# 2. Redistributions in binary form must reproduce the above copyright notice,
#    this list of conditions and the following disclaimer in the documentation
#    and/or other materials provided with the distribution.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
# ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
# DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
# ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
# (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
# ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

"""Unit tests for the SCC words"""

# pylint: disable=R0201,C0115,C0116,W0212

import unittest

from ttconv.scc.codes import SccChannel
from ttconv.scc.codes.control_codes import SccControlCode
from ttconv.scc.codes.extended_characters import SccExtendedCharacter
from ttconv.scc.codes.mid_row_codes import SccMidRowCode
from ttconv.scc.codes.preambles_address_codes import SccPreambleAddressCode
from ttconv.scc.word import SccWord


class SccWordTest(unittest.TestCase):

  def test_scc_word_is_hex_word(self):
    self.assertTrue(SccWord._is_hex_word("0000"))
    self.assertTrue(SccWord._is_hex_word("FFFF"))
    self.assertFalse(SccWord._is_hex_word("000"))
    self.assertFalse(SccWord._is_hex_word("12345"))
    self.assertFalse(SccWord._is_hex_word("GHIJ"))

  def test_scc_word_decipher_parity_bit(self):
    self.assertEqual(0b00000000, SccWord._decipher_parity_bit(0b10000000))
    self.assertEqual(0b00000010, SccWord._decipher_parity_bit(0b00000010))
    self.assertEqual(0b00001010, SccWord._decipher_parity_bit(0b10001010))

    translated_values = [
      0x80, 0x01, 0x02, 0x83, 0x04, 0x85, 0x86, 0x07, 0x08, 0x89, 0x8a, 0x0b, 0x8c, 0x0d, 0x0e, 0x8f,
      0x10, 0x91, 0x92, 0x13, 0x94, 0x15, 0x16, 0x97, 0x98, 0x19, 0x1a, 0x9b, 0x1c, 0x9d, 0x9e, 0x1f,
      0x20, 0xa1, 0xa2, 0x23, 0xa4, 0x25, 0x26, 0xa7, 0xa8, 0x29, 0x2a, 0xab, 0x2c, 0xad, 0xae, 0x2f,
      0xb0, 0x31, 0x32, 0xb3, 0x34, 0xb5, 0xb6, 0x37, 0x38, 0xb9, 0xba, 0x3b, 0xbc, 0x3d, 0x3e, 0xbf,
      0x40, 0xc1, 0xc2, 0x43, 0xc4, 0x45, 0x46, 0xc7, 0xc8, 0x49, 0x4a, 0xcb, 0x4c, 0xcd, 0xce, 0x4f,
      0xd0, 0x51, 0x52, 0xd3, 0x54, 0xd5, 0xd6, 0x57, 0x58, 0xd9, 0xda, 0x5b, 0xdc, 0x5d, 0x5e, 0xdf,
      0xe0, 0x61, 0x62, 0xe3, 0x64, 0xe5, 0xe6, 0x67, 0x68, 0xe9, 0xea, 0x6b, 0xec, 0x6d, 0x6e, 0xef,
      0x70, 0xf1, 0xf2, 0x73, 0xf4, 0x75, 0x76, 0xf7, 0xf8, 0x79, 0x7a, 0xfb, 0x7c, 0xfd, 0xfe, 0x7f
    ]

    for i in range(0x00, 0x80):
      self.assertEqual(i, SccWord._decipher_parity_bit(translated_values[i]))

  def test_scc_word_from_value(self):
    scc_word = SccWord.from_value(0x01)
    self.assertEqual((0x00, 0x01, 0x0001), (scc_word.byte_1, scc_word.byte_2, scc_word.value))
    scc_word = SccWord.from_value(0x0000)
    self.assertEqual((0x00, 0x00, 0x0000), (scc_word.byte_1, scc_word.byte_2, scc_word.value))
    scc_word = SccWord.from_value(0x1a2b)
    self.assertEqual((0x1a, 0x2b, 0x1a2b), (scc_word.byte_1, scc_word.byte_2, scc_word.value))
    scc_word = SccWord.from_value(0xffff)
    self.assertEqual((0x7f, 0x7f, 0x7f7f), (scc_word.byte_1, scc_word.byte_2, scc_word.value))

    self.assertRaises(ValueError, SccWord.from_value, 0x12345)

  def test_scc_word_from_bytes(self):
    scc_word = SccWord.from_bytes(0x00, 0x00)
    self.assertEqual((0x00, 0x00, 0x0000), (scc_word.byte_1, scc_word.byte_2, scc_word.value))
    scc_word = SccWord.from_bytes(0x1a, 0x2b)
    self.assertEqual((0x1a, 0x2b, 0x1a2b), (scc_word.byte_1, scc_word.byte_2, scc_word.value))
    scc_word = SccWord.from_bytes(0xff, 0xff)
    self.assertEqual((0x7f, 0x7f, 0x7f7f), (scc_word.byte_1, scc_word.byte_2, scc_word.value))

    self.assertRaises(ValueError, SccWord.from_bytes, 0x123, 0x45)
    self.assertRaises(ValueError, SccWord.from_bytes, 0x12, 0x345)

  def test_scc_word_from_str(self):
    scc_word = SccWord.from_str("0000")
    self.assertEqual((0x00, 0x00, 0x0000), (scc_word.byte_1, scc_word.byte_2, scc_word.value))
    scc_word = SccWord.from_str("1a2b")
    self.assertEqual((0x1a, 0x2b, 0x1a2b), (scc_word.byte_1, scc_word.byte_2, scc_word.value))
    scc_word = SccWord.from_str("ffff")
    self.assertEqual((0x7f, 0x7f, 0x7f7f), (scc_word.byte_1, scc_word.byte_2, scc_word.value))

    self.assertRaises(ValueError, SccWord.from_str, "ttml")
    self.assertRaises(ValueError, SccWord.from_str, "Hello")

  def test_scc_word_to_text(self):
    scc_word = SccWord.from_value(0x1234)
    self.assertEqual('\x12\x34', scc_word.to_text())
    scc_word = SccWord.from_value(0x01)
    self.assertEqual('\x01', scc_word.to_text())

    self.assertRaises(ValueError, SccWord.from_value, 0x01020304)

  def test_scc_word_get_code(self):
    self.assertEqual(SccControlCode.RCL, SccWord.from_str("9420").get_code())
    self.assertEqual(SccMidRowCode.ITALICS, SccWord.from_str("91ae").get_code())
    self.assertEqual(SccControlCode.BS, SccWord.from_str("9421").get_code())
    self.assertEqual(None, SccWord.from_str("4c6f").get_code())  # "Lo"
    self.assertEqual(None, SccWord.from_str("7265").get_code())  # "re"
    self.assertEqual(None, SccWord.from_str("6d20").get_code())  # "m "
    self.assertEqual(None, SccWord.from_str("6970").get_code())  # "ip"
    self.assertEqual(None, SccWord.from_str("7375").get_code())  # "su"
    self.assertEqual(None, SccWord.from_str("6d20").get_code())  # "m "
    self.assertEqual(SccExtendedCharacter.LATIN_CAPITAL_LETTER_A_WITH_ACUTE, SccWord.from_str("9220").get_code())
    self.assertEqual(SccControlCode.EDM, SccWord.from_str("942c").get_code())
    self.assertEqual(SccControlCode.EOC, SccWord.from_str("942f").get_code())
    self.assertEqual(SccControlCode.RU2, SccWord.from_str("9425").get_code())
    self.assertEqual(SccControlCode.CR, SccWord.from_str("94ad").get_code())
    self.assertEqual(SccPreambleAddressCode, SccWord.from_str("9673").get_code().__class__)
    self.assertEqual(None, SccWord.from_str("636f").get_code())  # "co"
    self.assertEqual(None, SccWord.from_str("6e73").get_code())  # "ns"
    self.assertEqual(None, SccWord.from_str("6563").get_code())  # "ec"
    self.assertEqual(None, SccWord.from_str("7465").get_code())  # "te"
    self.assertEqual(None, SccWord.from_str("7475").get_code())  # "tu"
    self.assertEqual(None, SccWord.from_str("7220").get_code())  # "r "
    self.assertEqual(None, SccWord.from_str("6164").get_code())  # "ad"
    self.assertEqual(None, SccWord.from_str("6970").get_code())  # "ip"
    self.assertEqual(None, SccWord.from_str("6973").get_code())  # "is"
    self.assertEqual(None, SccWord.from_str("6369").get_code())  # "ci"
    self.assertEqual(None, SccWord.from_str("6e67").get_code())  # "ng"
    self.assertEqual(None, SccWord.from_str("2065").get_code())  # " e"
    self.assertEqual(None, SccWord.from_str("6c69").get_code())  # "li"
    self.assertEqual(None, SccWord.from_str("742e").get_code())  # "t."

  def test_scc_word_get_channel(self):
    # invalid codes
    self.assertEqual(None, SccWord.from_value(0x1000).get_channel())
    self.assertEqual(None, SccWord.from_value(0x1432).get_channel())
    self.assertEqual(None, SccWord.from_value(0x1d00).get_channel())
    self.assertEqual(None, SccWord.from_value(0x1f38).get_channel())

    # Preamble Address Code
    self.assertEqual(SccChannel.CHANNEL_1, SccWord.from_value(0x91d0).get_channel())
    self.assertEqual(SccChannel.CHANNEL_2, SccWord.from_value(0x99d0).get_channel())

    # Attribute Code
    self.assertEqual(SccChannel.CHANNEL_1, SccWord.from_value(0x1020).get_channel())
    self.assertEqual(SccChannel.CHANNEL_2, SccWord.from_value(0x1820).get_channel())

  def test_scc_word_all_values(self):
    # check that all possible values decode to something
    for i in range(0x10000):
      scc_word = SccWord.from_value(i)
      scc_word.to_text()
      scc_word.get_code()
      scc_word.get_channel()



#!/usr/bin/env python
# -*- coding: UTF-8 -*-

# Copyright (c) 2023, Sandflow Consulting LLC
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
# 1. Redistributions of source code must retain the above copyright notice, this
#    list of conditions and the following disclaimer.
# 2. Redistributions in binary form must reproduce the above copyright notice,
#    this list of conditions and the following disclaimer in the documentation
#    and/or other materials provided with the distribution.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
# ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
# DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
# ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
# (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
# ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

"""SRT configuration"""

from __future__ import annotations

from dataclasses import dataclass, field
from ttconv.config import ModuleConfiguration

@dataclass
class SRTReaderConfiguration(ModuleConfiguration):
  """SRT reader configuration"""

  @classmethod
  def name(cls):
    return "srt_reader"

  # enables support for extended tags: {b}, {bold}, <bold> (and italic/underline equivalents)
  extended_tags: bool = field(default=False, metadata={"decoder": bool})

  # enables support for alignment tags: {\an1} through {\an9}
  alignment_tags: bool = field(default=False, metadata={"decoder": bool})

@dataclass
class SRTWriterConfiguration(ModuleConfiguration):
  """SRT writer configuration"""

  @classmethod
  def name(cls):
    return "srt_writer"

  # outputs text formatting tags
  text_formatting: bool = field(default=True, metadata={"decoder": bool})



#!/usr/bin/env python
# -*- coding: UTF-8 -*-

# Copyright (c) 2020, Sandflow Consulting LLC
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
# 1. Redistributions of source code must retain the above copyright notice, this
#    list of conditions and the following disclaimer.
# 2. Redistributions in binary form must reproduce the above copyright notice,
#    this list of conditions and the following disclaimer in the documentation
#    and/or other materials provided with the distribution.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
# ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
# DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
# ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
# (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
# ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

"""SCC utility functions"""
from numbers import Number
from typing import Union

from ttconv.model import CellResolutionType
from ttconv.style_properties import LengthType, CoordinateType, ExtentType


def get_position_from_offsets(x_offset: int, y_offset: int, units=LengthType.Units.c) -> CoordinateType:
  """Converts offsets into position"""
  x_position = LengthType(value=x_offset, units=units)
  y_position = LengthType(value=y_offset, units=units)

  return CoordinateType(x_position, y_position)


def get_extent_from_dimensions(width: Union[int, Number], height: Union[int, Number], units=LengthType.Units.c) -> ExtentType:
  """Converts dimensions into extent"""
  height = LengthType(value=height, units=units)
  width = LengthType(value=width, units=units)

  return ExtentType(height, width)


def convert_cells_to_percentages(dimensions: Union[CoordinateType, ExtentType], cell_resolution: CellResolutionType) -> Union[
  CoordinateType, ExtentType]:
  """Converts dimensions from cell units to percentage units"""
  if isinstance(dimensions, CoordinateType):
    if dimensions.x.units is not LengthType.Units.c or dimensions.y.units is not LengthType.Units.c:
      raise ValueError("Dimensions to convert must be in cell units")

    x_percent = round(dimensions.x.value * 100 / cell_resolution.columns)
    y_percent = round(dimensions.y.value * 100 / cell_resolution.rows)
    return get_position_from_offsets(x_percent, y_percent, LengthType.Units.pct)

  if isinstance(dimensions, ExtentType):
    if dimensions.width.units is not LengthType.Units.c or dimensions.height.units is not LengthType.Units.c:
      raise ValueError("Dimensions to convert must be in cell units")

    width_percent = round(dimensions.width.value * 100 / cell_resolution.columns)
    height_percent = round(dimensions.height.value * 100 / cell_resolution.rows)
    return get_extent_from_dimensions(width_percent, height_percent, LengthType.Units.pct)

  raise ValueError(f"Unsupported dimensions type: {dimensions.__class__.__name__}")



#!/usr/bin/env python
# -*- coding: UTF-8 -*-

# Copyright (c) 2020, Sandflow Consulting LLC
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
# 1. Redistributions of source code must retain the above copyright notice, this
#    list of conditions and the following disclaimer.
# 2. Redistributions in binary form must reproduce the above copyright notice,
#    this list of conditions and the following disclaimer in the documentation
#    and/or other materials provided with the distribution.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
# ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
# DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
# ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
# (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
# ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

"""Unit tests for the style property default values filter"""

# pylint: disable=R0201,C0115,C0116,W0212
from fractions import Fraction
import unittest

from ttconv.filters.isd.default_style_properties import DefaultStylePropertyValuesISDFilter
from ttconv.isd import ISD
from ttconv.model import P, ContentDocument, Region, Body, Div, Span, Text
from ttconv.style_properties import StyleProperties, NamedColors, FontStyleType, DirectionType


class DefaultStylesFilterTest(unittest.TestCase):

  def test_process_element(self):
    default_style_value_filter = DefaultStylePropertyValuesISDFilter({
      StyleProperties.Color: StyleProperties.Color.make_initial_value()
    })

    p = P()
    for style in StyleProperties.ALL:
      p.set_style(style, style.make_initial_value())

    self.assertEqual(len(StyleProperties.ALL), len(p._styles))
    self.assertEqual(StyleProperties.Color.make_initial_value(), p.get_style(StyleProperties.Color))

    default_style_value_filter._process_element(p)

    self.assertIsNone(p.get_style(StyleProperties.Color))
    self.assertEqual(len(StyleProperties.ALL) - 1, len(p._styles))

  def test_process_isd(self):
    default_style_value_filter = DefaultStylePropertyValuesISDFilter({
      StyleProperties.BackgroundColor: NamedColors.red.value,
      StyleProperties.Direction: DirectionType.ltr
    })

    doc = ContentDocument()

    r1 = Region("r1", doc)
    r1.set_style(StyleProperties.BackgroundColor, NamedColors.red.value)
    r1.set_style(StyleProperties.LuminanceGain, 2.0)
    doc.put_region(r1)

    b = Body(doc)
    b.set_begin(Fraction(1))
    b.set_end(Fraction(10))
    doc.set_body(b)

    div1 = Div(doc)
    div1.set_region(r1)
    b.push_child(div1)

    p1 = P(doc)
    p1.set_style(StyleProperties.BackgroundColor, NamedColors.white.value)
    p1.set_style(StyleProperties.Direction, DirectionType.rtl)
    div1.push_child(p1)

    span1 = Span(doc)
    span1.set_style(StyleProperties.BackgroundColor, NamedColors.red.value)
    span1.set_style(StyleProperties.FontStyle, FontStyleType.italic)
    span1.set_style(StyleProperties.Direction, DirectionType.ltr)
    p1.push_child(span1)

    t1 = Text(doc, "hello")
    span1.push_child(t1)

    significant_times = sorted(ISD.significant_times(doc))
    self.assertEqual(3, len(significant_times))

    isd = ISD.from_model(doc, significant_times[1])

    r1 = isd.get_region("r1")

    self.assertEqual(len(Region._applicableStyles), len(r1._styles))
    self.assertEqual(NamedColors.red.value, r1.get_style(StyleProperties.BackgroundColor))
    self.assertEqual(2.0, r1.get_style(StyleProperties.LuminanceGain))

    body1 = list(r1)[0]
    div1 = list(body1)[0]
    p1 = list(div1)[0]
    span1 = list(p1)[0]

    self.assertEqual(len(P._applicableStyles), len(p1._styles))
    self.assertEqual(NamedColors.white.value, p1.get_style(StyleProperties.BackgroundColor))
    self.assertEqual(DirectionType.rtl, p1.get_style(StyleProperties.Direction))

    self.assertEqual(len(Span._applicableStyles), len(span1._styles))
    self.assertEqual(NamedColors.red.value, span1.get_style(StyleProperties.BackgroundColor))
    self.assertEqual(FontStyleType.italic, span1.get_style(StyleProperties.FontStyle))
    self.assertEqual(DirectionType.ltr, span1.get_style(StyleProperties.Direction))

    default_style_value_filter.process(isd)

    self.assertEqual(len(Region._applicableStyles) - 1, len(r1._styles))
    self.assertIsNone(r1.get_style(StyleProperties.BackgroundColor))

    self.assertEqual(len(P._applicableStyles), len(p1._styles))
    self.assertEqual(NamedColors.white.value, p1.get_style(StyleProperties.BackgroundColor))
    self.assertEqual(DirectionType.rtl, p1.get_style(StyleProperties.Direction))

    self.assertEqual(len(Span._applicableStyles) - 1, len(span1._styles))
    self.assertIsNone(span1.get_style(StyleProperties.BackgroundColor))
    self.assertEqual(FontStyleType.italic, span1.get_style(StyleProperties.FontStyle))
    self.assertEqual(DirectionType.ltr, span1.get_style(StyleProperties.Direction))

if __name__ == '__main__':
  unittest.main()



#!/usr/bin/env python
# -*- coding: UTF-8 -*-

# Copyright (c) 2020, Sandflow Consulting LLC
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
# 1. Redistributions of source code must retain the above copyright notice, this
#    list of conditions and the following disclaimer.
# 2. Redistributions in binary form must reproduce the above copyright notice,
#    this list of conditions and the following disclaimer in the documentation
#    and/or other materials provided with the distribution.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
# ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
# DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
# ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
# (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
# ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

"""STL configuration"""

from __future__ import annotations
import typing

from dataclasses import dataclass, field
import re

from ttconv.config import ModuleConfiguration
import ttconv.style_properties as styles
from ttconv.imsc import utils
from ttconv.time_code import SmpteTimeCode

TC_PATTERN = re.compile(r"\d{2}[:;.,]\d{2}[:;.,]\d{2}[:;.,]\d{2}")
_MNR_PATTERN = re.compile("^\\d+$")

def _decode_font_stack(value: typing.Optional[str]) -> \
  typing.Optional[typing.Tuple[typing.Union[str, styles.GenericFontFamilyType]]]:
  return value if value is None else tuple(utils.parse_font_families(value))

def _decode_start_tc(value: typing.Optional[str]) -> typing.Optional[str]:
  if value is None:
    return None
  
  if value.upper() == "TCP":
    return "TCP"

  if TC_PATTERN.match(value):
    return value

  raise ValueError(f"Invalid start_tc '{value}' value. Expect: 'TCP' or 'HH:MM:SS:FF'.")

def _decode_max_row_count(value: typing.Optional[str]) -> typing.Optional[typing.Union[int, str]]:
  if value is None:
    return None
  
  if isinstance(value, str) and value.upper() == "MNR":
    return "MNR"

  if isinstance(value, int):
    return value

  raise ValueError(f"Invalid max_row_count '{value}' value. Expect: 'MNR' or integer.")

@dataclass
class STLReaderConfiguration(ModuleConfiguration):
  """STL reader configuration"""

  disable_fill_line_gap: bool = field(default=False, metadata={"decoder": bool})
  program_start_tc: typing.Optional[str] = field(default=None, metadata={"decoder": _decode_start_tc})
  disable_line_padding: bool = field(default=False, metadata={"decoder": bool})
  font_stack: typing.Optional[typing.Tuple[typing.Union[str, styles.GenericFontFamilyType]]] = \
                  field(default=None, metadata={"decoder": _decode_font_stack})
  max_row_count: typing.Optional[typing.Union[int, str]] = field(default=None, metadata={"decoder": _decode_max_row_count})

  @classmethod
  def name(cls):
    return "stl_reader"



#!/usr/bin/env python
# -*- coding: UTF-8 -*-

# Copyright (c) 2025, Sandflow Consulting LLC
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
# 1. Redistributions of source code must retain the above copyright notice, this
#    list of conditions and the following disclaimer.
# 2. Redistributions in binary form must reproduce the above copyright notice,
#    this list of conditions and the following disclaimer in the documentation
#    and/or other materials provided with the distribution.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
# ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
# DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
# ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
# (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
# ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

"""Unit tests for the IMSC 1.1 Text Profile filter.

Based on https://www.w3.org/TR/ttml-imsc1.1/ sections 6, 7.12, and 8.4.
"""

# pylint: disable=R0201,C0115,C0116

import logging
import os
import unittest
import io
from fractions import Fraction
import xml.etree.ElementTree as et

import ttconv.imsc.reader as imsc_reader
import ttconv.model as model
import ttconv.style_properties as styles
import ttconv.stl.reader as stl_reader
from ttconv.filters.doc.imsc11text import IMSC11TextFilter, IMSC11TextFilterConfig
from ttconv.filters.document_filter import DocumentFilter
from ttconv.imsc.designators import IMSC_11_TEXT_PROFILE_DESIGNATOR
from ttconv.vtt.reader import to_model


def _make_simple_doc():
  """Creates a minimal valid ContentDocument with Body > Div > P > Span > Text."""
  doc = model.ContentDocument()

  region = model.Region("r0", doc)
  region.set_style(
    styles.StyleProperties.Origin,
    styles.CoordinateType(
      x=styles.LengthType(0, styles.LengthType.Units.pct),
      y=styles.LengthType(0, styles.LengthType.Units.pct),
    ),
  )
  region.set_style(
    styles.StyleProperties.Extent,
    styles.ExtentType(
      width=styles.LengthType(100, styles.LengthType.Units.pct),
      height=styles.LengthType(100, styles.LengthType.Units.pct),
    ),
  )
  doc.put_region(region)

  body = model.Body(doc)
  doc.set_body(body)

  div = model.Div(doc)
  body.push_child(div)

  p = model.P(doc)
  p.set_begin(Fraction(0))
  p.set_end(Fraction(5))
  p.set_region(region)
  div.push_child(p)

  span = model.Span(doc)
  span.set_style(styles.StyleProperties.Color, styles.NamedColors.white.value)
  p.push_child(span)

  text = model.Text(doc, "Hello world")
  span.push_child(text)

  return doc


class IMSC11TextFilterTest(unittest.TestCase):

  def test_valid_document_passes(self):
    doc = _make_simple_doc()
    filt = IMSC11TextFilter()
    filt.process(doc)
    self.assertIn(IMSC_11_TEXT_PROFILE_DESIGNATOR, doc.get_content_profiles())

  def test_valid_vtt_document_passes(self):
    f = io.StringIO("WEBVTT\n\n00:00:00.000 --> 00:00:05.000\nHello world\n")
    doc = to_model(f)
    filt = IMSC11TextFilter()
    filt.process(doc)
    self.assertIn(IMSC_11_TEXT_PROFILE_DESIGNATOR, doc.get_content_profiles())

  def test_empty_document_passes(self):
    doc = model.ContentDocument()
    filt = IMSC11TextFilter()
    filt.process(doc)
    self.assertIn(IMSC_11_TEXT_PROFILE_DESIGNATOR, doc.get_content_profiles())

  def test_origin_px_pct_passes(self):
    doc = model.ContentDocument()
    region = model.Region("r0", doc)
    region.set_style(
      styles.StyleProperties.Origin,
      styles.CoordinateType(
        x=styles.LengthType(10, styles.LengthType.Units.px),
        y=styles.LengthType(10, styles.LengthType.Units.pct),
      ),
    )
    region.set_style(
      styles.StyleProperties.Extent,
      styles.ExtentType(
        width=styles.LengthType(100, styles.LengthType.Units.pct),
        height=styles.LengthType(100, styles.LengthType.Units.pct),
      ),
    )
    doc.put_region(region)

    filt = IMSC11TextFilter()
    filt.process(doc)
    self.assertIn(IMSC_11_TEXT_PROFILE_DESIGNATOR, doc.get_content_profiles())

  # Section 8.4.2: extent SHALL use px, %, or rh/rw
  def test_extent_rw_rh_passes(self):
    doc = model.ContentDocument()
    region = model.Region("r0", doc)
    region.set_style(
      styles.StyleProperties.Origin,
      styles.CoordinateType(
        x=styles.LengthType(0, styles.LengthType.Units.pct),
        y=styles.LengthType(0, styles.LengthType.Units.pct),
      ),
    )
    region.set_style(
      styles.StyleProperties.Extent,
      styles.ExtentType(
        width=styles.LengthType(80, styles.LengthType.Units.rw),
        height=styles.LengthType(80, styles.LengthType.Units.rh),
      ),
    )
    doc.put_region(region)

    filt = IMSC11TextFilter()
    filt.process(doc)
    self.assertIn(IMSC_11_TEXT_PROFILE_DESIGNATOR, doc.get_content_profiles())

  # Section 8.4.2: extent SHALL be present on all regions
  def test_region_without_extent_raises(self):
    doc = model.ContentDocument()
    region = model.Region("r0", doc)
    doc.put_region(region)

    with self.assertRaises(ValueError):
      IMSC11TextFilter().process(doc)

  def test_region_extent_from_initial_value_passes(self):
    doc = model.ContentDocument()
    doc.put_initial_value(
      styles.StyleProperties.Extent,
      styles.ExtentType(
        width=styles.LengthType(100, styles.LengthType.Units.pct),
        height=styles.LengthType(100, styles.LengthType.Units.pct),
      ),
    )
    region = model.Region("r0", doc)
    region.set_style(
      styles.StyleProperties.Origin,
      styles.CoordinateType(
        x=styles.LengthType(0, styles.LengthType.Units.pct),
        y=styles.LengthType(0, styles.LengthType.Units.pct),
      ),
    )
    doc.put_region(region)

    filt = IMSC11TextFilter()
    filt.process(doc)
    self.assertIn(IMSC_11_TEXT_PROFILE_DESIGNATOR, doc.get_content_profiles())

  # Section 8.4.12: linePadding only supports c units.
  # The model itself rejects non-c units on LinePadding at set_style() time.
  def test_line_padding_pct_rejected_by_model(self):
    doc = _make_simple_doc()
    for region in doc.iter_regions():
      with self.assertRaises(ValueError):
        region.set_style(
          styles.StyleProperties.LinePadding,
          styles.LengthType(5, styles.LengthType.Units.pct),
        )

  def test_line_padding_c_passes(self):
    doc = _make_simple_doc()
    for region in doc.iter_regions():
      region.set_style(
        styles.StyleProperties.LinePadding,
        styles.LengthType(0.5, styles.LengthType.Units.c),
      )

    filt = IMSC11TextFilter()
    filt.process(doc)
    self.assertIn(IMSC_11_TEXT_PROFILE_DESIGNATOR, doc.get_content_profiles())

  # Section 8.4.11: textShadow SHALL NOT have more than 4 shadows
  def test_text_shadow_5_shadows_raises(self):
    doc = _make_simple_doc()
    body = doc.get_body()
    with self.assertRaises(ValueError):
      for element in body.dfs_iterator():
        if isinstance(element, model.Span):
          shadow = styles.TextShadowType.Shadow(
            x_offset=styles.LengthType(1, styles.LengthType.Units.pct),
            y_offset=styles.LengthType(1, styles.LengthType.Units.pct),
          )
          element.set_style(
            styles.StyleProperties.TextShadow,
            styles.TextShadowType(shadows=(shadow, shadow, shadow, shadow, shadow)),
          )
        IMSC11TextFilter().process(doc)

  def test_text_shadow_4_shadows_passes(self):
    doc = _make_simple_doc()
    body = doc.get_body()
    for element in body.dfs_iterator():
      if isinstance(element, model.Span):
        shadow = styles.TextShadowType.Shadow(
          x_offset=styles.LengthType(1, styles.LengthType.Units.pct),
          y_offset=styles.LengthType(1, styles.LengthType.Units.pct),
        )
        element.set_style(
          styles.StyleProperties.TextShadow,
          styles.TextShadowType(shadows=(shadow, shadow, shadow, shadow)),
        )

    filt = IMSC11TextFilter()
    filt.process(doc)
    self.assertIn(IMSC_11_TEXT_PROFILE_DESIGNATOR, doc.get_content_profiles())

  # em units are permitted (#length-em is permitted in section 6)
  def test_em_unit_on_font_size_passes(self):
    doc = _make_simple_doc()
    body = doc.get_body()
    for element in body.dfs_iterator():
      if isinstance(element, model.P):
        element.set_style(
          styles.StyleProperties.FontSize,
          styles.LengthType(1.5, styles.LengthType.Units.em),
        )

    filt = IMSC11TextFilter()
    filt.process(doc)
    self.assertIn(IMSC_11_TEXT_PROFILE_DESIGNATOR, doc.get_content_profiles())

  def test_line_height_normal_passes(self):
    doc = _make_simple_doc()
    body = doc.get_body()
    for element in body.dfs_iterator():
      if isinstance(element, model.P):
        element.set_style(styles.StyleProperties.LineHeight, styles.SpecialValues.normal)

    filt = IMSC11TextFilter()
    filt.process(doc)
    self.assertIn(IMSC_11_TEXT_PROFILE_DESIGNATOR, doc.get_content_profiles())

  def test_first_violation_raises(self):
    doc = model.ContentDocument()
    region = model.Region("r0", doc)
    # No extent — violates section 8.4.2
    doc.put_region(region)

    with self.assertRaises(ValueError):
      IMSC11TextFilter().process(doc)

  # rh/rw units are allowed on all properties per maintainer review
  def test_rh_on_font_size_passes(self):
    doc = _make_simple_doc()
    body = doc.get_body()
    for element in body.dfs_iterator():
      if isinstance(element, model.P):
        element.set_style(
          styles.StyleProperties.FontSize,
          styles.LengthType(10, styles.LengthType.Units.rh),
        )

    filt = IMSC11TextFilter()
    filt.process(doc)
    self.assertIn(IMSC_11_TEXT_PROFILE_DESIGNATOR, doc.get_content_profiles())

  def test_rh_on_extent_passes(self):
    doc = model.ContentDocument()
    region = model.Region("r0", doc)
    region.set_style(
      styles.StyleProperties.Origin,
      styles.CoordinateType(
        x=styles.LengthType(0, styles.LengthType.Units.pct),
        y=styles.LengthType(0, styles.LengthType.Units.pct),
      ),
    )
    region.set_style(
      styles.StyleProperties.Extent,
      styles.ExtentType(
        width=styles.LengthType(100, styles.LengthType.Units.rw),
        height=styles.LengthType(100, styles.LengthType.Units.rh),
      ),
    )
    doc.put_region(region)

    filt = IMSC11TextFilter()
    filt.process(doc)
    self.assertIn(IMSC_11_TEXT_PROFILE_DESIGNATOR, doc.get_content_profiles())

  # Section 7.2.18: c units SHALL NOT be present outside of ebutts:linePadding
  def test_c_unit_on_font_size_raises(self):
    doc = _make_simple_doc()
    body = doc.get_body()
    for element in body.dfs_iterator():
      if isinstance(element, model.P):
        element.set_style(
          styles.StyleProperties.FontSize,
          styles.LengthType(1, styles.LengthType.Units.c),
        )

    with self.assertRaises(ValueError):
      IMSC11TextFilter().process(doc)

  # Section 8.4.13: ebutts:multiRowAlign SHALL only appear on p elements
  def test_multi_row_align_on_span_raises(self):
    doc = _make_simple_doc()
    body = doc.get_body()
    for element in body.dfs_iterator():
      if isinstance(element, model.Span):
        element.set_style(
          styles.StyleProperties.MultiRowAlign,
          styles.MultiRowAlignType.center,
        )

    with self.assertRaises(ValueError):
      IMSC11TextFilter().process(doc)

  def test_multi_row_align_on_p_passes(self):
    doc = _make_simple_doc()
    body = doc.get_body()
    for element in body.dfs_iterator():
      if isinstance(element, model.P):
        element.set_style(
          styles.StyleProperties.MultiRowAlign,
          styles.MultiRowAlignType.center,
        )

    filt = IMSC11TextFilter()
    filt.process(doc)
    self.assertIn(IMSC_11_TEXT_PROFILE_DESIGNATOR, doc.get_content_profiles())

  # Section 7.12.1.3: no more than 4 regions per ISD
  def test_more_than_4_regions_raises(self):
    doc = model.ContentDocument()
    regions = []
    for i in range(5):
      region = model.Region(f"r{i}", doc)
      region.set_style(
        styles.StyleProperties.Origin,
        styles.CoordinateType(
          x=styles.LengthType(0, styles.LengthType.Units.pct),
          y=styles.LengthType(i * 20, styles.LengthType.Units.pct),
        ),
      )
      region.set_style(
        styles.StyleProperties.Extent,
        styles.ExtentType(
          width=styles.LengthType(100, styles.LengthType.Units.pct),
          height=styles.LengthType(20, styles.LengthType.Units.pct),
        ),
      )
      doc.put_region(region)
      regions.append(region)

    body = model.Body(doc)
    doc.set_body(body)
    div = model.Div(doc)
    body.push_child(div)

    for region in regions:
      p = model.P(doc)
      p.set_begin(Fraction(0))
      p.set_end(Fraction(5))
      p.set_region(region)
      div.push_child(p)
      span = model.Span(doc)
      p.push_child(span)
      text = model.Text(doc, "Hello")
      span.push_child(text)

    with self.assertRaises(ValueError):
      IMSC11TextFilter().process(doc)

  def test_4_regions_passes(self):
    doc = model.ContentDocument()
    regions = []
    for i in range(4):
      region = model.Region(f"r{i}", doc)
      region.set_style(
        styles.StyleProperties.Origin,
        styles.CoordinateType(
          x=styles.LengthType(0, styles.LengthType.Units.pct),
          y=styles.LengthType(i * 25, styles.LengthType.Units.pct),
        ),
      )
      region.set_style(
        styles.StyleProperties.Extent,
        styles.ExtentType(
          width=styles.LengthType(100, styles.LengthType.Units.pct),
          height=styles.LengthType(25, styles.LengthType.Units.pct),
        ),
      )
      doc.put_region(region)
      regions.append(region)

    body = model.Body(doc)
    doc.set_body(body)
    div = model.Div(doc)
    body.push_child(div)

    for region in regions:
      p = model.P(doc)
      p.set_begin(Fraction(0))
      p.set_end(Fraction(5))
      p.set_region(region)
      div.push_child(p)
      span = model.Span(doc)
      p.push_child(span)
      text = model.Text(doc, "Hello")
      span.push_child(text)

    filt = IMSC11TextFilter()
    filt.process(doc)
    self.assertIn(IMSC_11_TEXT_PROFILE_DESIGNATOR, doc.get_content_profiles())

  # Section 8.4.10: textOutline thickness SHALL NOT exceed 10% of fontSize
  def test_text_outline_exceeds_10pct_raises(self):
    doc = _make_simple_doc()
    body = doc.get_body()
    for element in body.dfs_iterator():
      if isinstance(element, model.Span):
        element.set_style(
          styles.StyleProperties.FontSize,
          styles.LengthType(100, styles.LengthType.Units.pct),
        )
        element.set_style(
          styles.StyleProperties.TextOutline,
          styles.TextOutlineType(
            thickness=styles.LengthType(11, styles.LengthType.Units.pct),
          ),
        )

    with self.assertRaises(ValueError):
      IMSC11TextFilter().process(doc)

  def test_text_outline_within_10pct_passes(self):
    doc = _make_simple_doc()
    body = doc.get_body()
    for element in body.dfs_iterator():
      if isinstance(element, model.Span):
        element.set_style(
          styles.StyleProperties.FontSize,
          styles.LengthType(100, styles.LengthType.Units.pct),
        )
        element.set_style(
          styles.StyleProperties.TextOutline,
          styles.TextOutlineType(
            thickness=styles.LengthType(10, styles.LengthType.Units.pct),
          ),
        )

    filt = IMSC11TextFilter()
    filt.process(doc)
    self.assertIn(IMSC_11_TEXT_PROFILE_DESIGNATOR, doc.get_content_profiles())

  # Section 8.4.7/8.4.8: document-wide origin/position mutual exclusion
  def test_origin_position_coexist_across_regions_raises(self):
    doc = model.ContentDocument()
    r0 = model.Region("r0", doc)
    r0.set_style(
      styles.StyleProperties.Origin,
      styles.CoordinateType(
        x=styles.LengthType(0, styles.LengthType.Units.pct),
        y=styles.LengthType(0, styles.LengthType.Units.pct),
      ),
    )
    r0.set_style(
      styles.StyleProperties.Extent,
      styles.ExtentType(
        width=styles.LengthType(50, styles.LengthType.Units.pct),
        height=styles.LengthType(50, styles.LengthType.Units.pct),
      ),
    )
    doc.put_region(r0)

    r1 = model.Region("r1", doc)
    r1.set_style(
      styles.StyleProperties.Position,
      styles.PositionType(
        h_edge=styles.PositionType.HEdge.left,
        h_offset=styles.LengthType(50, styles.LengthType.Units.pct),
        v_edge=styles.PositionType.VEdge.top,
        v_offset=styles.LengthType(0, styles.LengthType.Units.pct),
      ),
    )
    r1.set_style(
      styles.StyleProperties.Extent,
      styles.ExtentType(
        width=styles.LengthType(50, styles.LengthType.Units.pct),
        height=styles.LengthType(50, styles.LengthType.Units.pct),
      ),
    )
    doc.put_region(r1)

    with self.assertRaises(ValueError):
      IMSC11TextFilter().process(doc)

  def test_all_regions_use_origin_passes(self):
    doc = model.ContentDocument()
    for i in range(2):
      r = model.Region(f"r{i}", doc)
      r.set_style(
        styles.StyleProperties.Origin,
        styles.CoordinateType(
          x=styles.LengthType(i * 50, styles.LengthType.Units.pct),
          y=styles.LengthType(0, styles.LengthType.Units.pct),
        ),
      )
      r.set_style(
        styles.StyleProperties.Extent,
        styles.ExtentType(
          width=styles.LengthType(50, styles.LengthType.Units.pct),
          height=styles.LengthType(50, styles.LengthType.Units.pct),
        ),
      )
      doc.put_region(r)

    filt = IMSC11TextFilter()
    filt.process(doc)
    self.assertIn(IMSC_11_TEXT_PROFILE_DESIGNATOR, doc.get_content_profiles())

  # Section 8.4.7: tts:origin SHALL use px or % only
  def test_origin_rw_raises(self):
    doc = model.ContentDocument()
    region = model.Region("r0", doc)
    region.set_style(
      styles.StyleProperties.Origin,
      styles.CoordinateType(
        x=styles.LengthType(10, styles.LengthType.Units.rw),
        y=styles.LengthType(10, styles.LengthType.Units.pct),
      ),
    )
    region.set_style(
      styles.StyleProperties.Extent,
      styles.ExtentType(
        width=styles.LengthType(80, styles.LengthType.Units.pct),
        height=styles.LengthType(80, styles.LengthType.Units.pct),
      ),
    )
    doc.put_region(region)

    with self.assertRaises(ValueError):
      IMSC11TextFilter().process(doc)

  # Section 8.4.12: linePadding SHALL use c units only
  def test_line_padding_rh_raises(self):
    doc = _make_simple_doc()
    for region in doc.iter_regions():
      region.set_style(
        styles.StyleProperties.LinePadding,
        styles.LengthType(0.5, styles.LengthType.Units.rh),
      )

    with self.assertRaises(ValueError):
      IMSC11TextFilter().process(doc)

  def test_line_padding_rw_raises(self):
    doc = _make_simple_doc()
    for region in doc.iter_regions():
      region.set_style(
        styles.StyleProperties.LinePadding,
        styles.LengthType(0.5, styles.LengthType.Units.rw),
      )

    with self.assertRaises(ValueError):
      IMSC11TextFilter().process(doc)

  # Spatial overlap checks

  def _make_two_region_doc(self, ox1, oy1, ew1, eh1, ox2, oy2, ew2, eh2):
    """Builds a doc with two regions each containing active content.

    All coordinates and sizes are in percent units.
    """
    doc = model.ContentDocument()
    body = model.Body(doc)
    doc.set_body(body)
    div = model.Div(doc)
    body.push_child(div)

    for idx, (ox, oy, ew, eh) in enumerate(
        [(ox1, oy1, ew1, eh1), (ox2, oy2, ew2, eh2)]
    ):
      region = model.Region(f"r{idx}", doc)
      region.set_style(
        styles.StyleProperties.Origin,
        styles.CoordinateType(
          x=styles.LengthType(ox, styles.LengthType.Units.pct),
          y=styles.LengthType(oy, styles.LengthType.Units.pct),
        ),
      )
      region.set_style(
        styles.StyleProperties.Extent,
        styles.ExtentType(
          width=styles.LengthType(ew, styles.LengthType.Units.pct),
          height=styles.LengthType(eh, styles.LengthType.Units.pct),
        ),
      )
      doc.put_region(region)

      p = model.P(doc)
      p.set_begin(Fraction(0))
      p.set_end(Fraction(5))
      p.set_region(region)
      div.push_child(p)
      span = model.Span(doc)
      p.push_child(span)
      text = model.Text(doc, "Hello")
      span.push_child(text)

    return doc

  def test_overlapping_regions_raises(self):
    # r0: x=[0,60], y=[0,50]   r1: x=[40,100], y=[0,50]  → overlap at x=[40,60]
    doc = self._make_two_region_doc(0, 0, 60, 50, 40, 0, 60, 50)
    with self.assertRaises(ValueError):
      IMSC11TextFilter().process(doc)

  def test_abutting_regions_horizontal_passes(self):
    # r0: x=[0,50], y=[0,100]   r1: x=[50,100], y=[0,100]  → share edge, no overlap
    doc = self._make_two_region_doc(0, 0, 50, 100, 50, 0, 50, 100)
    filt = IMSC11TextFilter()
    filt.process(doc)
    self.assertIn(IMSC_11_TEXT_PROFILE_DESIGNATOR, doc.get_content_profiles())

  def test_abutting_regions_vertical_passes(self):
    # r0: x=[0,100], y=[0,50]   r1: x=[0,100], y=[50,100]  → share edge, no overlap
    doc = self._make_two_region_doc(0, 0, 100, 50, 0, 50, 100, 50)
    filt = IMSC11TextFilter()
    filt.process(doc)
    self.assertIn(IMSC_11_TEXT_PROFILE_DESIGNATOR, doc.get_content_profiles())

  def test_non_overlapping_regions_with_gap_passes(self):
    # r0: x=[0,40], y=[0,100]   r1: x=[60,100], y=[0,100]  → gap at x=[40,60]
    doc = self._make_two_region_doc(0, 0, 40, 100, 60, 0, 40, 100)
    filt = IMSC11TextFilter()
    filt.process(doc)
    self.assertIn(IMSC_11_TEXT_PROFILE_DESIGNATOR, doc.get_content_profiles())

  # Filter auto-registration
  def test_filter_registered_by_name(self):
    filt_cls = DocumentFilter.get_filter_by_name("imsc11text")
    self.assertIs(filt_cls, IMSC11TextFilter)

  def test_stl_docs(self):
    for root, _subdirs, files in os.walk("src/test/resources/stl"):
      for filename in files:
        (name, ext) = os.path.splitext(filename)
        if ext == ".stl":
            with self.subTest(name):
              with open(os.path.join(root, filename), "rb") as i_file:
                model = stl_reader.to_model(i_file)
                self.assertIsNotNone(model)
                filt = IMSC11TextFilter()
                filt.process(model)

  def test_imsc_1_test_suite(self):
    for root, _subdirs, files in os.walk("src/test/resources/ttml/imsc-tests/imsc1/ttml"):
      for filename in files:
        (name, ext) = os.path.splitext(filename)
        if ext == ".ttml":
            with self.subTest(name):
              logging.getLogger().info("*****dummy*****") # dummy log
              tree = et.parse(os.path.join(root, filename))
              model = imsc_reader.to_model(tree)
              self.assertIsNotNone(model)
              filt = IMSC11TextFilter()
              filt.process(model)


  def test_imsc_1_1_test_suite(self):
    for root, _subdirs, files in os.walk("src/test/resources/ttml/imsc-tests/imsc1_1/ttml"):
      for filename in files:
        (name, ext) = os.path.splitext(filename)
        if ext == ".ttml":
            with self.subTest(name):
              logging.getLogger().info("*****dummy*****") # dummy log
              tree = et.parse(os.path.join(root, filename))
              model = imsc_reader.to_model(tree)
              self.assertIsNotNone(model)
              filt = IMSC11TextFilter()
              filt.process(model)

  def test_imsc_1_3_test_suite(self):
    for root, _subdirs, files in os.walk("src/test/resources/ttml/imsc-tests/imsc1_3/ttml"):
      for filename in files:
        (name, ext) = os.path.splitext(filename)
        if ext == ".ttml":
            with self.subTest(name):
              tree = et.parse(os.path.join(root, filename))
              model = imsc_reader.to_model(tree)
              self.assertIsNotNone(model)
              filt = IMSC11TextFilter()
              with self.assertRaises(ValueError):
                filt.process(model)

if __name__ == "__main__":
  unittest.main()



#!/usr/bin/env python
# -*- coding: UTF-8 -*-

# Copyright (c) 2020, Sandflow Consulting LLC
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
# 1. Redistributions of source code must retain the above copyright notice, this
#    list of conditions and the following disclaimer.
# 2. Redistributions in binary form must reproduce the above copyright notice,
#    this list of conditions and the following disclaimer in the documentation
#    and/or other materials provided with the distribution.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
# ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
# DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
# ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
# (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
# ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

'''Unit tests for the IMSC reader'''

# pylint: disable=R0201,C0115,C0116

import unittest
import xml.etree.ElementTree as et
import os
import logging
from fractions import Fraction
from ttconv.imsc import namespaces
from ttconv.imsc.attributes import DropMode, DropModeAttribute, TimeBase, TimeBaseAttribute, TimeContainerAttribute
import ttconv.model as model
import ttconv.style_properties as styles
import ttconv.imsc.reader as imsc_reader
import ttconv.imsc.style_properties as imsc_styles
from ttconv.time_code import SmpteTimeCode

class IMSCReaderTest(unittest.TestCase):

  def test_reader_tt_element_not_root_element(self):

    xml_str = """<?xml version="1.0" encoding="UTF-8"?>
      <not_tt xml:lang="en"
          xmlns="http://www.w3.org/ns/ttml"
          xmlns:ttm="http://www.w3.org/ns/ttml#metadata" 
          xmlns:tts="http://www.w3.org/ns/ttml#styling"
          xmlns:ttp="http://www.w3.org/ns/ttml#parameter" 
          xmlns:ittp="http://www.w3.org/ns/ttml/profile/imsc1#parameter"
          ittp:activeArea="50% 50% 80% 80%"
          tts:extent="640px 480px"
          ttp:profile="http://www.w3.org/ns/ttml/profile/imsc1/text">
      </not_tt>"""

    tt_not_root = et.ElementTree(et.fromstring(xml_str))
    self.assertIsNone(imsc_reader.to_model(tt_not_root))

  def test_body_only(self):
    tree = et.parse('src/test/resources/ttml/body_only.ttml')
    imsc_reader.to_model(tree)

  def test_basic_time_containment_001(self):
    tree = et.parse('src/test/resources/ttml/imsc-tests/imsc1/ttml/timing/BasicTimeContainment001.ttml')
    doc = imsc_reader.to_model(tree)

    body = doc.get_body()

    self.assertIsNone(body.get_begin())
    self.assertEqual(body.get_end(), Fraction(10))

    div = list(body)[0]

    self.assertIsNone(div.get_begin())
    self.assertEqual(div.get_end(), Fraction(10))

    p = list(div)[0]

    self.assertIsNone(p.get_begin())
    self.assertIsNone(p.get_end())

    span_children = [v for v in list(p) if len(list(v)[0].get_text().strip()) > 0]

    self.assertIsNone(span_children[0].get_begin())
    self.assertEqual(span_children[0].get_end(), Fraction(5))

    self.assertIsNone(span_children[1].get_begin())
    self.assertEqual(span_children[1].get_end(), Fraction(10))

  def test_basic_time_containment_002(self):
    tree = et.parse('src/test/resources/ttml/imsc-tests/imsc1/ttml/timing/BasicTimeContainment002.ttml')
    doc = imsc_reader.to_model(tree)

    body = doc.get_body()

    self.assertIsNone(body.get_begin())
    self.assertEqual(body.get_end(), Fraction(20))

    div = list(body)[0]

    self.assertIsNone(div.get_begin())
    self.assertEqual(div.get_end(), Fraction(20))

    p_children = [v for v in list(div) if isinstance(v, model.P)]

    self.assertIsNone(p_children[0].get_begin())
    self.assertEqual(p_children[0].get_end(), Fraction(10))

    self.assertEqual(p_children[1].get_begin(), Fraction(10))
    self.assertEqual(p_children[1].get_end(), Fraction(20))

  def test_basic_time_containment_003(self):
    tree = et.parse('src/test/resources/ttml/imsc-tests/imsc1/ttml/timing/BasicTimeContainment003.ttml')
    doc = imsc_reader.to_model(tree)

    body = doc.get_body()

    self.assertIsNone(body.get_begin())
    self.assertEqual(body.get_end(), Fraction(10))

    div = list(body)[0]

    self.assertIsNone(div.get_begin())
    self.assertEqual(div.get_end(), Fraction(10))

    p_children = [v for v in list(div) if isinstance(v, model.P)]

    self.assertIsNone(p_children[0].get_begin())
    self.assertEqual(p_children[0].get_end(), Fraction(10))

    self.assertEqual(p_children[1].get_begin(), Fraction(10))
    self.assertEqual(p_children[1].get_end(), Fraction(20))

    span_children = [v for v in list(p_children[0]) if len(list(v)[0].get_text().strip()) > 0]

    self.assertEqual(span_children[0].get_begin(), Fraction(5))
    self.assertEqual(span_children[0].get_end(), Fraction(10))

    self.assertEqual(span_children[1].get_begin(), Fraction(10))
    self.assertEqual(span_children[1].get_end(), Fraction(15))

  def test_basic_timing_007(self):
    tree = et.parse('src/test/resources/ttml/imsc-tests/imsc1/ttml/timing/BasicTiming007.ttml')
    doc = imsc_reader.to_model(tree)

    body = doc.get_body()

    self.assertIsNone(body.get_begin())
    self.assertEqual(body.get_end(), Fraction(20))

    div = list(body)[0]

    self.assertIsNone(div.get_begin())
    self.assertEqual(div.get_end(), Fraction(20))

    p = list(div)[0]

    self.assertEqual(p.get_begin(), 5)
    self.assertEqual(p.get_end(), 20)

    span_children = [v for v in list(p) if isinstance(v, model.Span)]

    self.assertEqual(len(span_children), 1)

    self.assertEqual(span_children[0].get_begin(), None)
    self.assertEqual(span_children[0].get_end(), Fraction(10))

  def test_imsc_1_test_suite(self):
    for root, _subdirs, files in os.walk("src/test/resources/ttml/imsc-tests/imsc1/ttml"):
      for filename in files:
        (name, ext) = os.path.splitext(filename)
        if ext == ".ttml":
          with self.subTest(name), self.assertLogs() as logs:
            logging.getLogger().info("*****dummy*****") # dummy log
            tree = et.parse(os.path.join(root, filename))
            self.assertIsNotNone(imsc_reader.to_model(tree))
            if len(logs.output) > 1:
              self.fail(logs.output)

  def test_imsc_1_1_test_suite(self):
    for root, _subdirs, files in os.walk("src/test/resources/ttml/imsc-tests/imsc1_1/ttml"):
      for filename in files:
        (name, ext) = os.path.splitext(filename)
        if ext == ".ttml":
          with self.subTest(name):
            tree = et.parse(os.path.join(root, filename))
            self.assertIsNotNone(imsc_reader.to_model(tree))

  def test_imsc_1_3_test_suite(self):
    for root, _subdirs, files in os.walk("src/test/resources/ttml/imsc-tests/imsc1_3/ttml"):
      for filename in files:
        (name, ext) = os.path.splitext(filename)
        if ext == ".ttml":
          with self.subTest(name):
            tree = et.parse(os.path.join(root, filename))
            self.assertIsNotNone(imsc_reader.to_model(tree))

  def test_referential_styling(self):
    tree = et.parse('src/test/resources/ttml/referential_styling.ttml')
    doc = imsc_reader.to_model(tree)

    divs = list(doc.get_body())

    self.assertEqual(divs[0].get_style(styles.StyleProperties.Color), styles.NamedColors.green.value)
    self.assertEqual(divs[0].get_style(styles.StyleProperties.BackgroundColor), styles.NamedColors.blue.value)

    self.assertEqual(divs[1].get_style(styles.StyleProperties.Color), styles.NamedColors.black.value)
    self.assertEqual(divs[1].get_style(styles.StyleProperties.BackgroundColor), styles.NamedColors.blue.value)

    regions = list(doc.iter_regions())

    self.assertEqual(regions[0].get_style(styles.StyleProperties.Color), styles.NamedColors.blue.value)
    self.assertEqual(regions[0].get_style(styles.StyleProperties.BackgroundColor), styles.NamedColors.yellow.value)

    self.assertEqual(regions[1].get_style(styles.StyleProperties.Color), styles.NamedColors.red.value)
    self.assertEqual(regions[1].get_style(styles.StyleProperties.BackgroundColor), styles.NamedColors.yellow.value)
  
  def test_initial(self):
    tree = et.parse('src/test/resources/ttml/imsc-tests/imsc1_1/ttml/initial/initial002.ttml')
    doc = imsc_reader.to_model(tree)

    self.assertEqual(doc.get_initial_value(styles.StyleProperties.Color), styles.NamedColors.green.value)
    self.assertEqual(doc.get_initial_value(styles.StyleProperties.FontStyle), styles.FontStyleType.italic)

  def test_frame_rate(self):
    tree = et.parse('src/test/resources/ttml/imsc-tests/imsc1/ttml/timing/TimeExpressions001.ttml')
    doc = imsc_reader.to_model(tree)

    # <p begin="0s" end="24f">24f = 1.001s</p>
    p = list(list(doc.get_body())[0])[3]

    self.assertEqual(p.get_end(), Fraction(4394201, 1000))

  def test_ooo_set_element(self):
    xml_str = """<?xml version="1.0" encoding="UTF-8"?>
      <tt xml:lang="en"
          xmlns="http://www.w3.org/ns/ttml"
          xmlns:tts="http://www.w3.org/ns/ttml#styling">
      <body>
        <div><p>hello</p></div>
        <set tts:color="red"/>
        <div><p>bye</p></div>
      </body>
      </tt>"""

    tree = et.ElementTree(et.fromstring(xml_str))

    with self.assertLogs() as logs:
      logging.getLogger().info("*****dummy*****") # dummy log
      self.assertIsNotNone(imsc_reader.to_model(tree))
      if len(logs.output) != 2:
        self.fail(logs.output)

  def test_text_emphasis(self):
    value = imsc_styles.StyleProperties.TextEmphasis.extract(None, "dot after")
    self.assertEqual(value.style, styles.TextEmphasisType.Style.filled_dot)
    self.assertEqual(value.position, styles.TextEmphasisType.Position.after)

    value = imsc_styles.StyleProperties.TextEmphasis.extract(None, "dot before")
    self.assertEqual(value.style, styles.TextEmphasisType.Style.filled_dot)
    self.assertEqual(value.position, styles.TextEmphasisType.Position.before)

    value = imsc_styles.StyleProperties.TextEmphasis.extract(None, "filled after")
    self.assertEqual(value.style, styles.TextEmphasisType.Style.filled_circle)
    self.assertEqual(value.position, styles.TextEmphasisType.Position.after)

    value = imsc_styles.StyleProperties.TextEmphasis.extract(None, "open before")
    self.assertEqual(value.style, styles.TextEmphasisType.Style.open_circle)
    self.assertEqual(value.position, styles.TextEmphasisType.Position.before)

  def test_cell_resolution(self):
    xml_str = """<?xml version="1.0" encoding="UTF-8"?>
    <tt xml:lang="en"
        xmlns="http://www.w3.org/ns/ttml"
        xmlns:ttp="http://www.w3.org/ns/ttml#parameter"
        ttp:cellResolution="32 15">
    </tt>"""
    doc = imsc_reader.to_model(et.ElementTree(et.fromstring(xml_str)))
    self.assertEqual(doc.get_cell_resolution().columns, 32)
    self.assertEqual(doc.get_cell_resolution().rows, 15)

  def test_content_profiles(self):
    xml_str = """<?xml version="1.0" encoding="UTF-8"?>
    <tt xml:lang="en"
        xmlns="http://www.w3.org/ns/ttml"
        xmlns:ttp="http://www.w3.org/ns/ttml#parameter"
        ttp:contentProfiles="http://www.w3.org/ns/ttml/profile/imsc1.1/text http://www.w3.org/ns/ttml/profile/imsc1/text">
    </tt>"""
    doc = imsc_reader.to_model(et.ElementTree(et.fromstring(xml_str)))
    self.assertSetEqual(doc.get_content_profiles(), {"http://www.w3.org/ns/ttml/profile/imsc1.1/text", "http://www.w3.org/ns/ttml/profile/imsc1/text"})

  def test_timeBase_parameter(self):
    self.assertEqual(TimeBaseAttribute.extract(et.Element("tt", {f"{{{namespaces.TTP}}}timeBase": "media"})), TimeBase.media)
    self.assertEqual(TimeBaseAttribute.extract(et.Element("tt", {f"{{{namespaces.TTP}}}timeBase": "smpte"})), TimeBase.smpte)
    self.assertEqual(TimeBaseAttribute.extract(et.Element("tt")), TimeBase.media)

    with self.assertRaises(ValueError):
      TimeBaseAttribute.extract(et.Element("tt", {f"{{{namespaces.TTP}}}timeBase": "clock"}))

    with self.assertLogs() as logs:
      logging.getLogger().info("*****dummy*****") # dummy log
      self.assertEqual(TimeBaseAttribute.extract(et.Element("tt", {f"{{{namespaces.TTP}}}timeBase": "x"})), TimeBase.media)
      if len(logs.output) != 2:
        self.fail(logs.output)

  def test_dropframe_parameter(self):
    self.assertEqual(DropModeAttribute.extract(et.Element("tt", {f"{{{namespaces.TTP}}}dropMode": "nonDrop"})), DropMode.nonDrop)
    self.assertEqual(DropModeAttribute.extract(et.Element("tt", {f"{{{namespaces.TTP}}}dropMode": "dropNTSC"})), DropMode.dropNTSC)
    self.assertEqual(DropModeAttribute.extract(et.Element("tt")), DropMode.nonDrop)

    with self.assertRaises(ValueError):
      DropModeAttribute.extract(et.Element("tt", {f"{{{namespaces.TTP}}}dropMode": "dropPAL"}))

    with self.assertLogs() as logs:
      logging.getLogger().info("*****dummy*****") # dummy log
      self.assertEqual(DropModeAttribute.extract(et.Element("tt", {f"{{{namespaces.TTP}}}dropMode": "x"})), DropMode.nonDrop)
      if len(logs.output) != 2:
        self.fail(logs.output)

  def test_smpte_tc_nondrop(self):
    xml_str = """<?xml version="1.0" encoding="UTF-8"?>
    <tt xml:lang="en"
        xmlns="http://www.w3.org/ns/ttml"
        xmlns:ttp="http://www.w3.org/ns/ttml#parameter"
        ttp:frameRate="30" ttp:frameRateMultiplier="1000 1001" ttp:timeBase="smpte"
    >
    <body begin="01:02:03:20"/>
    </tt>"""
    doc = imsc_reader.to_model(et.ElementTree(et.fromstring(xml_str)))
    body = doc.get_body()
    self.assertEqual(body.get_begin(), (3723 * 30 + 20)/Fraction(30000, 1001))

  def test_smpte_tc_drop(self):
    xml_str = """<?xml version="1.0" encoding="UTF-8"?>
    <tt xml:lang="en"
        xmlns="http://www.w3.org/ns/ttml"
        xmlns:ttp="http://www.w3.org/ns/ttml#parameter"
        ttp:frameRate="30" ttp:frameRateMultiplier="1000 1001" ttp:timeBase="smpte" ttp:dropMode="dropNTSC"
    >
    <body begin="01:02:03:20"/>
    </tt>"""
    doc = imsc_reader.to_model(et.ElementTree(et.fromstring(xml_str)))
    body = doc.get_body()
    self.assertEqual(body.get_begin(), SmpteTimeCode(1, 2, 3, 20, Fraction(30000, 1001), True).to_temporal_offset())

if __name__ == '__main__':
  unittest.main()
